@@ -32,9 +32,7 @@ from extras import safe_hasattr, try_import, try_imports
 parse_mime_type = try_import('mimeparse.parse_mime_type')
 Queue = try_imports(['Queue.Queue', 'queue.Queue'])
 
-from pyrsistent import PClass, field, pmap_field, pset_field, pmap, pset, thaw
-
-from testtools.compat import str_is_unicode, text_or_bytes, _u, _b
+from testtools.compat import str_is_unicode, _u, _b
 from testtools.content import (
     Content,
     text_content,
@@ -634,41 +632,48 @@ class StreamTagger(CopyStreamResult):
         super(StreamTagger, self).status(*args, **kwargs)
 
 
-class _TestRecord(PClass):
+class _TestRecord(object):
     """Representation of a test."""
 
-    """The test id."""
-    id = field(text_or_bytes, mandatory=True)
+    def __init__(self, id, tags, details, status, timestamps):
+        # The test id.
+        self.id = id
 
-    """Tags for the test."""
-    tags = pset_field(text_or_bytes, optional=False)
+        # Tags for the test.
+        self.tags = tags
 
-    """File attachments."""
-    # XXX: Documentation says these are unicode, but tests pass in str.
-    details = pmap_field(text_or_bytes, Content, optional=False)
+        # File attachments.
+        self.details = details
 
-    """One of the StreamResult status codes."""
-    status = field(
-        text_or_bytes, mandatory=True,
-        invariant=lambda x: (x in STATES, 'Invalid state'))
+        # One of the StreamResult status codes.
+        self.status = status
 
-    """Pair of timestamps (x, y).
-
-    x is the first timestamp we received for this test, y is the one that
-    triggered the notification. y can be None if the test hanged.
-    """
-    timestamps = field(tuple, mandatory=True)
+        # Pair of timestamps (x, y).
+        # x is the first timestamp we received for this test, y is the one that
+        # triggered the notification. y can be None if the test hanged.
+        self.timestamps = timestamps
 
     @classmethod
     def create(cls, test_id, timestamp):
         return cls(
             id=test_id,
-            tags=pset(),
-            details=pmap(),
+            tags=set(),
+            details={},
             status='unknown',
             timestamps=(timestamp, None),
         )
 
+    def set(self, *args, **kwargs):
+        if args:
+            setattr(self, args[0], args[1])
+        for key, value in kwargs.items():
+            setattr(self, key, value)
+        return self
+
+    def transform(self, data, value):
+        getattr(self, data[0])[data[1]] = value
+        return self
+
     def to_dict(self):
         """Convert record into a "test dict".
 
@@ -688,8 +693,8 @@ class _TestRecord(PClass):
         """
         return {
             'id': self.id,
-            'tags': thaw(self.tags),
-            'details': thaw(self.details),
+            'tags': self.tags,
+            'details': self.details,
             'status': self.status,
             'timestamps': list(self.timestamps),
         }
@@ -733,8 +738,8 @@ class _TestRecord(PClass):
         return PlaceHolder(
             self.id,
             outcome=outcome,
-            details=thaw(self.details),
-            tags=thaw(self.tags),
+            details=self.details,
+            tags=self.tags,
             timestamps=self.timestamps,
         )
 
@@ -760,7 +765,7 @@ def _make_content_type(mime_type=None):
     return ContentType(primary, sub, parameters)
 
 
-_status_map = pmap({
+_status_map = {
     'inprogress': 'addFailure',
     'unknown': 'addFailure',
     'success': 'addSuccess',
@@ -768,7 +773,7 @@ _status_map = pmap({
     'fail': 'addFailure',
     'xfail': 'addExpectedFailure',
     'uxsuccess': 'addUnexpectedSuccess',
-})
+}
 
 
 class _StreamToTestRecord(StreamResult):
@@ -1349,6 +1354,8 @@ class ExtendedToOriginalDecorator(object):
         self._tags = TagContext()
         # Only used for old TestResults that do not have failfast.
         self._failfast = False
+        # Used for old TestResults that do not have stop.
+        self._shouldStop = False
 
     def __repr__(self):
         return '<%s %r>' % (self.__class__.__name__, self.decorated)
@@ -1485,9 +1492,15 @@ class ExtendedToOriginalDecorator(object):
             return
         return method(offset, whence)
 
-    @property
-    def shouldStop(self):
-        return self.decorated.shouldStop
+    def _get_shouldStop(self):
+        return getattr(self.decorated, 'shouldStop', self._shouldStop)
+
+    def _set_shouldStop(self, value):
+        if safe_hasattr(self.decorated, 'shouldStop'):
+            self.decorated.shouldStop = value
+        else:
+            self._shouldStop = value
+    shouldStop = property(_get_shouldStop, _set_shouldStop)
 
     def startTest(self, test):
         self._tags = TagContext(self._tags)
@@ -1501,7 +1514,10 @@ class ExtendedToOriginalDecorator(object):
             return
 
     def stop(self):
-        return self.decorated.stop()
+        method = getattr(self.decorated, 'stop', None)
+        if method:
+            return method()
+        self.shouldStop = True
 
     def stopTest(self, test):
         self._tags = self._tags.parent
